@@ -3,16 +3,16 @@
 # Copyright 2005-2009 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Library General Public
+# modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
-# version 2 of the License, or (at your option) any later version.
+# version 2.1 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Library General Public License for more details.
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU Library General Public
+# You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
@@ -37,8 +37,8 @@ log = logging.getLogger("braille")
 import signal
 import os
 
-import gobject
-gobject.threads_init()
+from gi.repository import GObject
+GObject.threads_init()
 
 try:
     import louis
@@ -56,6 +56,8 @@ except:
     _brlAPIAvailable = False
     _brlAPIRunning = False
 
+import settings
+
 try:
     # This can fail due to gtk not being available.  We want to
     # be able to recover from that if possible.  The main driver
@@ -64,22 +66,22 @@ try:
     #
     import brlmon
 except:
-    pass
+    settings.enableBrailleMonitor = False
+
 import debug
 import eventsynthesizer
 import orca_state
-import settings
 
 # Right now, the orca autogen.sh/configure needs a priori knowledge of
 # where the liblouis tables are.  When running autogen.sh/configure,
-# platform.py:tablesdir will be set to point to the liblouis table
+# orca_platform.py:tablesdir will be set to point to the liblouis table
 # location.  If not found, it will be the empty string.  We need to
 # capture that error condition, otherwise braille contraction will
 # just plain fail.  See also bgo#610134.  [[TODO: WDW - see if the
 # liblouis bindings can give us the tablesdir information at runtime
 # http://code.google.com/p/liblouis/issues/detail?id=9]]
 #
-from platform import tablesdir
+from orca_platform import tablesdir
 if louis and not tablesdir:
     debug.println(debug.LEVEL_SEVERE,
                   "Contraction tables for liblouis cannot be found.")
@@ -351,9 +353,14 @@ class Region:
 
         self.expandOnCursor = expandOnCursor
 
+        try:
+            string = string.decode("UTF-8")
+        except UnicodeEncodeError:
+            pass
+
         # The uncontracted string for the line.
         #
-        self.rawLine = string.decode("UTF-8").strip("\n")
+        self.rawLine = string.strip("\n")
 
         if self.contracted:
             self.contractionTable = settings.brailleContractionTable or \
@@ -521,6 +528,13 @@ class Component(Region):
         associated with this region.  Note that the zeroeth character may have
         been scrolled off the display."""
 
+        if orca_state.activeScript and orca_state.activeScript.utilities.\
+           grabFocusBeforeRouting(self.accessible, offset):
+            try:
+                self.accessible.queryComponent().grabFocus()
+            except:
+                pass
+
         try:
             action = self.accessible.queryAction()
         except:
@@ -595,11 +609,20 @@ class Text(Region):
             self.caretOffset = 0
             self.lineOffset = 0
 
-        string = string.decode("UTF-8")
+        try:
+            string = string.decode("UTF-8")
+        except UnicodeEncodeError:
+            pass
         if label:
-            label = label.decode("UTF-8")
+            try:
+                label = label.decode("UTF-8")
+            except UnicodeEncodeError:
+                pass
         if eol:
-            eol = eol.decode("UTF-8")
+            try:
+                eol = eol.decode("UTF-8")
+            except UnicodeEncodeError:
+                pass
 
         try:
             endOffset = endOffset - self.lineOffset
@@ -699,7 +722,8 @@ class Text(Region):
         if caretOffset < 0:
             return
 
-        orca_state.activeScript.setCaretOffset(self.accessible, caretOffset)
+        orca_state.activeScript.utilities.setCaretOffset(
+            self.accessible, caretOffset)
 
     def getAttributeMask(self, getLinkMask=True):
         """Creates a string which can be used as the attrOr field of brltty's
@@ -748,14 +772,14 @@ class Text(Region):
                 n += 1
 
         if attrIndicator:
-            enabledAttributes = script.attributeStringToDictionary(
+            keys, enabledAttributes = script.utilities.stringToKeysAndDict(
                 settings.enabledBrailledTextAttributes)
 
             offset = self.lineOffset
             while offset < lineEndOffset:
                 attributes, startOffset, endOffset = \
-                            script.getTextAttributes(self.accessible,
-                                                     offset, True)
+                    script.utilities.textAttributes(self.accessible,
+                                                    offset, True)
                 if endOffset <= offset:
                     break
                 mask = settings.TEXT_ATTR_BRAILLE_NONE
@@ -772,7 +796,7 @@ class Text(Region):
                         regionMask[i] |= attrIndicator
 
         if selIndicator:
-            selections = script.getAllTextSelections(self.accessible)
+            selections = script.utilities.allTextSelections(self.accessible)
             for startOffset, endOffset in selections:
                 maskStart = max(startOffset - self.lineOffset, 0)
                 maskEnd = min(endOffset - self.lineOffset, stringLength)
@@ -879,7 +903,8 @@ class ReviewText(Region):
         been scrolled off the display."""
 
         caretOffset = self.getCaretOffset(offset)
-        orca_state.activeScript.setCaretOffset(self.accessible, caretOffset)
+        orca_state.activeScript.utilities.setCaretOffset(
+            self.accessible, caretOffset)
 
 class Line:
     """A horizontal line on the display.  Each Line is composed of a sequential
@@ -1358,12 +1383,23 @@ def refresh(panToCursor=True,
         cursorCell += 1 # Normalize to 1-based offset
 
     logLine = "BRAILLE LINE:  '%s'" % string
+    try:
+        logLine = logLine.encode("UTF-8")
+    except UnicodeDecodeError:
+        pass
+
     debug.println(debug.LEVEL_INFO, logLine)
-    log.info(logLine.encode("UTF-8"))
+    log.info(logLine)
+
     logLine = "     VISIBLE:  '%s', cursor=%d" % \
                     (string[startPos:endPos], cursorCell)
+    try:
+        logLine = logLine.encode("UTF-8")
+    except UnicodeDecodeError:
+        pass
+
     debug.println(debug.LEVEL_INFO, logLine)
-    log.info(logLine.encode("UTF-8"))
+    log.info(logLine)
 
     substring = string[startPos:endPos]
     if not _brlAPIRunning:
@@ -1411,13 +1447,18 @@ def refresh(panToCursor=True,
 
     if settings.enableBrailleMonitor:
         if not _monitor:
-            _monitor = brlmon.BrlMon(_displaySize[0])
-            _monitor.show_all()
+            try:
+                _monitor = brlmon.BrlMon(_displaySize[0])
+                _monitor.show_all()
+            except:
+                debug.println(debug.LEVEL_WARNING, "brlmon failed")
+                _monitor = None
         if attributeMask:
             subMask = attributeMask[startPos:endPos]
         else:
             subMask = None
-        _monitor.writeText(cursorCell, substring, subMask)
+        if _monitor:
+            _monitor.writeText(cursorCell, substring, subMask)
     elif _monitor:
         _monitor.destroy()
         _monitor = None
@@ -1455,7 +1496,7 @@ def killFlash(restoreSaved=True):
     global viewport
     if _flashEventSourceId:
         if _flashEventSourceId > 0:
-            gobject.source_remove(_flashEventSourceId)
+            GObject.source_remove(_flashEventSourceId)
         if restoreSaved:
             (_lines, _regionWithFocus, viewport, flashTime) = _saved
             refresh(panToCursor=False, stopFlash=False)
@@ -1464,9 +1505,9 @@ def killFlash(restoreSaved=True):
 def resetFlashTimer():
     global _flashEventSourceId
     if _flashEventSourceId > 0:
-        gobject.source_remove(_flashEventSourceId)
+        GObject.source_remove(_flashEventSourceId)
         flashTime = _saved[3]
-        _flashEventSourceId = gobject.timeout_add(flashTime, _flashCallback)
+        _flashEventSourceId = GObject.timeout_add(flashTime, _flashCallback)
 
 def _initFlash(flashTime):
     """Sets up the state needed to flash a message or clears any existing
@@ -1485,13 +1526,13 @@ def _initFlash(flashTime):
 
     if _flashEventSourceId:
         if _flashEventSourceId > 0:
-            gobject.source_remove(_flashEventSourceId)
+            GObject.source_remove(_flashEventSourceId)
         _flashEventSourceId = 0
     else:
         _saved = (_lines, _regionWithFocus, viewport, flashTime)
 
     if flashTime > 0:
-        _flashEventSourceId = gobject.timeout_add(flashTime, _flashCallback)
+        _flashEventSourceId = GObject.timeout_add(flashTime, _flashCallback)
     elif flashTime < 0:
         _flashEventSourceId = -666
 
@@ -1544,6 +1585,16 @@ def displayMessage(message, cursor=-1, flashTime=0):
     setFocus(region)
     refresh(True, stopFlash=False)
 
+def displayKeyEvent(event):
+    """Displays a KeyboardEvent. Typically reserved for locking keys like
+    Caps Lock and Num Lock."""
+
+    lockingStateString = event.getLockingStateString()
+    if lockingStateString:
+        keyname = event.getKeyName()
+        msg = "%s %s" % (keyname, lockingStateString)
+        displayMessage(msg, flashTime=settings.brailleFlashTime)
+
 def panLeft(panAmount=0):
     """Pans the display to the left, limiting the pan to the beginning
     of the line being displayed.
@@ -1773,8 +1824,8 @@ def init(callback=None, tty=7):
             debug.println(\
                 debug.LEVEL_CONFIGURATION,
                 "Braille module has been initialized using tty=%d" % tty)
-        _brlAPISourceId = gobject.io_add_watch(_brlAPI.fileDescriptor,
-                                               gobject.IO_IN,
+        _brlAPISourceId = GObject.io_add_watch(_brlAPI.fileDescriptor,
+                                               GObject.IO_IN,
                                                _brlAPIKeyReader)
     except:
         debug.println(debug.LEVEL_CONFIGURATION,
@@ -1817,7 +1868,7 @@ def shutdown():
 
     if _brlAPIRunning:
         _brlAPIRunning = False
-        gobject.source_remove(_brlAPISourceId)
+        GObject.source_remove(_brlAPISourceId)
         _brlAPISourceId = 0
         try:
             _brlAPI.leaveTtyMode()
