@@ -3,16 +3,16 @@
 # Copyright 2005-2009 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Library General Public
+# modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
-# version 2 of the License, or (at your option) any later version.
+# version 2.1 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Library General Public License for more details.
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU Library General Public
+# You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
@@ -26,15 +26,19 @@ __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import pyatspi
+from gi.repository import Atspi, Atk
 
 import braille
 import generator
+import orca
 import orca_state
-import rolenames
 import settings
 
+from braille_rolenames import shortRoleNames
 from orca_i18n import ngettext  # for ngettext support
 
+_settingsManager = getattr(orca, '_settingsManager')
+
 class Space:
     """A dummy class to indicate we want to insert a space into an
     utterance, but only if there is text prior to the space."""
@@ -88,17 +92,21 @@ class BrailleGenerator(generator.Generator):
         except:
             focusedRegion = None
         for region in result:
+            try:
+                role = obj.getRole()
+            except:
+                role = None
             if isinstance(region, (braille.Component, braille.Text)) \
                and region.accessible == obj:
                 focusedRegion = region
                 break
             elif isinstance(region, braille.Text) \
-                 and obj.getRole() == pyatspi.ROLE_COMBO_BOX \
+                 and role == pyatspi.ROLE_COMBO_BOX \
                  and region.accessible.parent == obj:
                 focusedRegion = region
                 break
             elif isinstance(region, braille.Component) \
-                 and obj.getRole() == pyatspi.ROLE_TABLE_CELL \
+                 and role == pyatspi.ROLE_TABLE_CELL \
                  and region.accessible.parent == obj:
                 focusedRegion = region
                 break
@@ -119,14 +127,45 @@ class BrailleGenerator(generator.Generator):
         """
         result = []
         role = args.get('role', obj.getRole())
-        if (role in settings.brailleForceRoles)\
-           or ((settings.brailleVerbosityLevel \
+        if (role in _settingsManager.getSetting('brailleForceRoles'))\
+           or ((_settingsManager.getSetting('brailleVerbosityLevel') \
                 == settings.VERBOSITY_LEVEL_VERBOSE)\
                and not args.get('readingRow', False)\
                and (role != pyatspi.ROLE_UNKNOWN)):
-            result.append(rolenames.getBrailleForRoleName(obj, role))
+            result.append(self.getLocalizedRoleName(obj, role))
         return result
 
+    @staticmethod
+    def getLocalizedRoleName(obj, role=None):
+        """Returns the localized name of the given Accessible object; the name
+        is suitable to be brailled.
+
+        Arguments:
+        - obj: an Accessible object
+        - role: an optional pyatspi role to use instead
+        """
+
+        if _settingsManager.getSetting('brailleRolenameStyle') \
+                == settings.BRAILLE_ROLENAME_STYLE_SHORT:
+            objRole = role or obj.getRole()
+            rv = shortRoleNames.get(objRole)
+            if rv:
+                return rv
+
+        if not isinstance(role, pyatspi.Role):
+            try:
+                return obj.getLocalizedRoleName()
+            except:
+                return ''
+
+        if not role:
+            return ''
+
+        nonlocalized = Atspi.role_get_name(role)
+        atkRole = Atk.role_for_name(nonlocalized)
+
+        return Atk.role_get_localized_name(atkRole)
+
     #####################################################################
     #                                                                   #
     # Keyboard shortcut information                                     #
@@ -157,8 +196,11 @@ class BrailleGenerator(generator.Generator):
         I wonder if this string should be moved to settings.py.]]]
         """
         result = []
-        alertAndDialogCount = \
-            self._script.utilities.unfocusedAlertAndDialogCount(obj)
+        try:
+            alertAndDialogCount = \
+                self._script.utilities.unfocusedAlertAndDialogCount(obj)
+        except:
+            alertAndDialogCount = 0
         if alertAndDialogCount > 0:
             # Translators: this tells the user how many unfocused
             # alert and dialog windows plus the total number of
@@ -181,7 +223,7 @@ class BrailleGenerator(generator.Generator):
         previous object with focus.
         """
         result = []
-        if not settings.enableBrailleContext:
+        if not _settingsManager.getSetting('enableBrailleContext'):
             return result
         args['includeContext'] = False
 
@@ -205,8 +247,11 @@ class BrailleGenerator(generator.Generator):
             # page tab lists might be a nice thing to include. Logged
             # as bugzilla bug 319751.]]]
             #
-            role = parent.getRole()
-            if role != pyatspi.ROLE_FILLER \
+            try:
+                role = parent.getRole()
+            except:
+                role = None
+            if role and role != pyatspi.ROLE_FILLER \
                 and role != pyatspi.ROLE_SECTION \
                 and role != pyatspi.ROLE_SPLIT_PANE \
                 and role != pyatspi.ROLE_DESKTOP_FRAME \
@@ -221,8 +266,7 @@ class BrailleGenerator(generator.Generator):
             # container for the grouped objects.  When we detect this,
             # we add the label to the overall context.]]]
             #
-            if parent.getRole() in [pyatspi.ROLE_FILLER,
-                                    pyatspi.ROLE_PANEL]:
+            if role in [pyatspi.ROLE_FILLER, pyatspi.ROLE_PANEL]:
                 label = self._script.utilities.displayedLabel(parent)
                 if label and len(label) and not label.isspace():
                     if not excludeRadioButtonGroup:
@@ -250,7 +294,10 @@ class BrailleGenerator(generator.Generator):
         details.
         """
         result = []
-        relations = obj.getRelationSet()
+        try:
+            relations = obj.getRelationSet()
+        except:
+            relations = []
         for relation in relations:
             if relation.getRelationType() ==  pyatspi.RELATION_LABELLED_BY:
                 labelledBy = relation.getTarget(0)
@@ -293,7 +340,7 @@ class BrailleGenerator(generator.Generator):
         # are on the very first line.  Otherwise, we show only the
         # line.
         #
-        include = settings.enableBrailleContext
+        include = _settingsManager.getSetting('enableBrailleContext')
         if not include:
             return include
         try:
@@ -302,9 +349,12 @@ class BrailleGenerator(generator.Generator):
             text = None
         if text and (self._script.utilities.isTextArea(obj) \
                      or (obj.getRole() in [pyatspi.ROLE_LABEL])):
-            [lineString, startOffset, endOffset] = text.getTextAtOffset(
-                text.caretOffset,
-                pyatspi.TEXT_BOUNDARY_LINE_START)
+            try:
+                [lineString, startOffset, endOffset] = text.getTextAtOffset(
+                    text.caretOffset, pyatspi.TEXT_BOUNDARY_LINE_START)
+            except:
+                return include
+
             include = startOffset == 0
             if include:
                 for relation in obj.getRelationSet():
@@ -322,7 +372,7 @@ class BrailleGenerator(generator.Generator):
 
     def _generateEol(self, obj, **args):
         result = []
-        if not settings.disableBrailleEOL:
+        if not _settingsManager.getSetting('disableBrailleEOL'):
             if not args.get('mode', None):
                 args['mode'] = self._mode
             args['stringType'] = 'eol'
@@ -348,6 +398,11 @@ class BrailleGenerator(generator.Generator):
             while content and isinstance(content[-1], Space):
                 content = content[0:-1]
             for element in content:
+                try:
+                    element = element.decode("UTF-8")
+                except UnicodeEncodeError:
+                    pass
+
                 if isinstance(element, Space) and prior:
                     combined += element.delimiter
                     prior = None
