@@ -26,10 +26,6 @@ __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import orca.scripts.default as default
-try:
-    import orca.gsmag as mag
-except:
-    import orca.mag as mag
 import orca.speech as speech
 import pyatspi
 
@@ -88,15 +84,25 @@ class Script(default.Script):
         found = False
         for app in self.utilities.knownApplications():
             i = 0
-            while i < app.childCount:
-                win = app.getChildAtIndex(i)
+            try:
+                childCount = app.childCount
+            except:
+                continue
+            while i < childCount:
+                try:
+                    win = app.getChildAtIndex(i)
+                except:
+                    win = None
                 if win is None:
                     print "app error " + app.name
                 elif win.name == objName:
                     found = True
                 i = i + 1
 
-        text = obj.name
+        try:
+            text = obj.name
+        except:
+            text = objName
 
         # Translators: the "Workspace " and "Desk " strings are
         # the prefix of what metacity shows when you press
@@ -115,7 +121,6 @@ class Script(default.Script):
 
         self.displayBrailleMessage(text)
         speech.speak(text)
-        mag.magnifyAccessible(None, obj)
 
     def onNameChanged(self, event):
         """The status bar in metacity tells us what toplevel window
