@@ -3,16 +3,16 @@
 # Copyright 2008-2009 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Library General Public
+# modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
-# version 2 of the License, or (at your option) any later version.
+# version 2.1 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Library General Public License for more details.
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU Library General Public
+# You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
@@ -207,7 +207,7 @@ class TutorialGenerator:
         """
 
         utterances = []
-        name = self._script.getDisplayedText(obj)
+        name = self._script.utilities.displayedText(obj)
         if not name and obj.description:
             name = obj.description
 
@@ -217,8 +217,11 @@ class TutorialGenerator:
 
         # If this application has more than one unfocused alert or
         # dialog window, tell user how to give them focus.
-        alertAndDialogCount = \
-                    self._script.getUnfocusedAlertAndDialogCount(obj)
+        try:
+            alertAndDialogCount = \
+                self._script.utilities.unfocusedAlertAndDialogCount(obj)
+        except:
+            alertAndDialogCount = 0
         if alertAndDialogCount > 0:
             utterances.append(childWindowsMsg)
 
@@ -268,7 +271,7 @@ class TutorialGenerator:
         """
 
         utterances = []
-        name = self._script.getDisplayedText(obj)
+        name = self._script.utilities.displayedText(obj)
         if not name and obj.description:
             name = obj.description
 
@@ -283,7 +286,10 @@ class TutorialGenerator:
         desktopMsg = _("To get to the system menus press the alt+f1 key.")
 
         scriptName = self._script.name
-        sibling = obj.parent.getChildAtIndex(0)
+        try:
+            sibling = obj.parent.getChildAtIndex(0)
+        except AttributeError:
+            sibling = None
         if 'nautilus' in scriptName and obj == sibling:
             utterances.append(desktopMsg)
 
@@ -385,7 +391,7 @@ class TutorialGenerator:
         msg = _("Type in text.")
 
         if (not alreadyFocused or forceTutorial) and \
-           not self._script.isReadOnlyTextArea(obj):
+           not self._script.utilities.isReadOnlyTextArea(obj):
             utterances.append(msg)
 
         self._debugGenerator("_getTutorialForText",
@@ -603,9 +609,9 @@ class TutorialGenerator:
             except NotImplementedError:
                 parent_table = None
             if settings.readTableCellRow and parent_table \
-                and (not self._script.isLayoutOnly(obj.parent)):
+                and not self._script.utilities.isLayoutOnly(obj.parent):
                 parent = obj.parent
-                index = self._script.getCellIndex(obj)
+                index = self._script.utilities.cellIndex(obj)
                 row = parent_table.getRowAtIndex(index)
                 column = parent_table.getColumnAtIndex(index)
 
